#include <stdio.h>
#include <stdlib.h>
#include <imsls.h>

int main() {
    int   n_groups = 3;
    int   nrow, nvar, ncol, i, nrmiss;
    float *x, *xtemp;
    float *prior_out, *means, *cov, *coef;
    float *table, *d2, *stats, *prob;
    int   *counts, *cm;
    static int perm[5] = {1, 2, 3, 4, 0};

    /* Retrieve the Fisher Iris Data Set */
    xtemp = imsls_f_data_sets(3, IMSLS_N_OBSERVATIONS, &nrow,
        IMSLS_N_VARIABLES, &ncol, 0);
    nvar = ncol - 1;

    /* Move the group column to end of the the matrix */
    x = imsls_f_permute_matrix(nrow, ncol, xtemp, perm, 
        IMSLS_PERMUTE_COLUMNS, 0);
    imsls_free(xtemp);

    prior_out = (float *) malloc(n_groups*sizeof(float));
    counts    = (int *)   malloc(n_groups*sizeof(int));
    means     = (float *) malloc(n_groups*nvar*sizeof(float));
    cov       = (float *) malloc(nvar*nvar*(ngroups+1)*sizeof(float));
    coef      = (float *) malloc(n_groups*(nvar+1)*sizeof(float));
    table     = (float *) malloc(n_groups*n_groups*sizeof(float));
    d2        = (float *) malloc(n_groups*n_groups*sizeof(float));
    stats     = (float *) malloc((4+2*(n_groups+1))*sizeof(float));
    cm        = (int *)   malloc(nrow*sizeof(int));
    prob      = (float *) malloc(nrow*n_groups*sizeof(float));

    /*Initialize Analysis*/
    imsls_f_discriminant_analysis (0, nvar, x, n_groups, 
         IMSLS_IDO, 1,
         IMSLS_METHOD, 2, 0);

    /*Add In Each Observation*/
    for (i=0;i<nrow;i=i+1) {
      imsls_f_discriminant_analysis (1, nvar, (x+i*ncol), n_groups, 
         IMSLS_IDO, 2, 0);
    }

    /*Remove observation 0 from the analysis */
    imsls_f_discriminant_analysis (1, nvar, (x+0), n_groups, 
         IMSLS_ROWS_DELETE,
         IMSLS_IDO, 2, 0);

    /*Add observation 0 back into the analysis */
    imsls_f_discriminant_analysis (1, nvar, (x+0), n_groups, 
         IMSLS_IDO, 2, 0);

    /*Compute statistics*/
    imsls_f_discriminant_analysis (0, nvar, x, n_groups, 
         IMSLS_PRIOR_PROPORTIONAL,
         IMSLS_PRIOR_OUTPUT_USER, prior_out,
         IMSLS_IDO, 3, 0);

    imsls_f_write_matrix("Prior OUT", 1, n_groups, prior_out, 0); 

    /*Classify One observation at a time, using proportional priors*/
    for (i=0;i<nrow;i=i+1) {
      imsls_f_discriminant_analysis (1, nvar, (x+i*ncol), n_groups, 
         IMSLS_IDO, 4,
         IMSLS_CLASS_MEMBERSHIP_USER, (cm+i),
         IMSLS_PROB_USER, (prob+i*n_groups), 0);
    }

    /*Compute covariance matrices and release internal workspace*/
    imsls_f_discriminant_analysis (0, nvar, x, n_groups, 
         IMSLS_IDO, 5, 
         IMSLS_COV_USER, cov, 
         IMSLS_GROUP_COUNTS_USER, counts,
         IMSLS_COEF_USER, coef,
         IMSLS_MEANS_USER, means,
         IMSLS_STATS_USER, stats,
         IMSLS_CLASS_TABLE_USER, table,
         IMSLS_MAHALANOBIS_USER, d2,
         IMSLS_N_ROWS_MISSING, &nrmiss, 0);

    imsls_i_write_matrix("Counts", 1, n_groups, counts, 0);
    imsls_f_write_matrix("Coef", n_groups, nvar+1, coef, 0);
    imsls_f_write_matrix("Means", n_groups, nvar, means, 0);
    imsls_f_write_matrix("Stats", 12, 1, stats, 0);
    imsls_i_write_matrix("Membership", 1, nrow, cm, 0);
    imsls_f_write_matrix("Table", n_groups, n_groups, table, 0);
    imsls_f_write_matrix("Prob", nrow, n_groups, prob, 0);
    imsls_f_write_matrix("D2", n_groups, n_groups, d2, 0);  
    imsls_f_write_matrix("Covariance", nvar, nvar, cov, 0);
    printf("\nnrmiss = %3d\n", nrmiss);

    imsls_free(means);
    imsls_free(stats);
    imsls_free(counts);
    imsls_free(coef);
    imsls_free(cm);
    imsls_free(table);
    imsls_free(prob);
    imsls_free(d2);
    imsls_free(prior_out);
    imsls_free(cov);
 
}
